"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.trySaveUpmConfig = exports.makeUpmConfigLoader = exports.makeUpmConfigDirGetter = exports.RequiredEnvMissingError = void 0;
const path_1 = __importDefault(require("path"));
const toml_1 = __importDefault(require("@iarna/toml"));
const ts_custom_error_1 = require("ts-custom-error");
const ts_results_es_1 = require("ts-results-es");
const file_io_1 = require("./file-io");
const env_util_1 = require("../utils/env-util");
const string_parsing_1 = require("../utils/string-parsing");
const wsl_1 = require("./wsl");
const configFileName = ".upmconfig.toml";
class RequiredEnvMissingError extends ts_custom_error_1.CustomError {
    _class = "RequiredEnvMissingError";
    constructor(...keyNames) {
        super(`Env was required to contain a value for one of the following keys, but all were missing: ${keyNames
            .map((keyName) => `"${keyName}"`)
            .join(", ")}.`);
    }
}
exports.RequiredEnvMissingError = RequiredEnvMissingError;
/**
 * Makes a {@link GetUpmConfigDir} function.
 */
function makeUpmConfigDirGetter(getHomePath) {
    return (wsl, systemUser) => {
        const systemUserSubPath = "Unity/config/ServiceAccounts";
        if (wsl) {
            if (systemUser)
                return (0, wsl_1.tryGetWslPath)("ALLUSERSPROFILE").map((it) => path_1.default.join(it, systemUserSubPath));
            return (0, wsl_1.tryGetWslPath)("USERPROFILE");
        }
        if (systemUser) {
            const profilePath = (0, env_util_1.tryGetEnv)("ALLUSERSPROFILE");
            if (profilePath === null)
                return (0, ts_results_es_1.Err)(new RequiredEnvMissingError("ALLUSERSPROFILE")).toAsyncResult();
            return (0, ts_results_es_1.Ok)(path_1.default.join(profilePath, systemUserSubPath)).toAsyncResult();
        }
        return getHomePath().toAsyncResult();
    };
}
exports.makeUpmConfigDirGetter = makeUpmConfigDirGetter;
/**
 * Makes a {@link LoadUpmConfig} function.
 */
function makeUpmConfigLoader(readFile) {
    return (directory) => {
        const configPath = path_1.default.join(directory, configFileName);
        return (readFile(configPath)
            .andThen(string_parsing_1.tryParseToml)
            // TODO: Actually validate
            .map((toml) => toml)
            .orElse((error) => error instanceof file_io_1.NotFoundError ? (0, ts_results_es_1.Ok)(null) : (0, ts_results_es_1.Err)(error)));
    };
}
exports.makeUpmConfigLoader = makeUpmConfigLoader;
/**
 * Save the upm config.
 * @param config The config to save.
 * @param configDir The directory in which to save the config.
 * @returns The path to which the file was saved.
 */
const trySaveUpmConfig = (writeFile, config, configDir) => {
    const configPath = path_1.default.join(configDir, configFileName);
    const content = toml_1.default.stringify(config);
    return writeFile(configPath, content).map(() => configPath);
};
exports.trySaveUpmConfig = trySaveUpmConfig;
//# sourceMappingURL=upm-config-io.js.map